package cn.wisenergy.chnmuseum.party.web.controller;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.ZipUtil;
import cn.wisenergy.chnmuseum.party.common.dfs.FastDFSUtils;
import cn.wisenergy.chnmuseum.party.common.video.VideoEncryptUtil;
import cn.wisenergy.chnmuseum.party.common.vo.GenericPageParam;
import cn.wisenergy.chnmuseum.party.model.Asset;
import cn.wisenergy.chnmuseum.party.service.AssetService;
import cn.wisenergy.chnmuseum.party.web.controller.base.BaseController;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.tobato.fastdfs.domain.fdfs.MetaData;
import com.sun.xml.internal.messaging.saaj.util.ByteOutputStream;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresAuthentication;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * <pre>
 * 文件资产 前端控制器
 * </pre>
 *
 * @author Danny Lee
 * @since 2021-04-06
 */
@Slf4j
@RestController
@RequestMapping("/asset")
@Api(tags = {"文件资产操作接口"})
public class AssetController extends BaseController {

    private static final String BASE_STRING = "1234567890abcdef";

    @Resource
    private AssetService assetService;

    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "_index", value = "分页起始偏移量", paramType = "query", dataType = "Integer"),
            @ApiImplicitParam(name = "_size", value = "返回条数", paramType = "query", dataType = "Integer"),
            @ApiImplicitParam(name = "nameOrCode", value = "视频文件名", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "videoContentCatId", value = "视频内容分类ID", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "videoContentCopyrightOwnerId", value = "视频内容版权方ID", paramType = "query", dataType = "String")
    })
    @PostMapping("/getPageList")
    @RequiresAuthentication  //@RequiresPermissions("asset:page")
    @ApiOperation(value = "获取视频汇出分页列表", notes = "获取视频汇出分页列表")
    public Map<String, Object> getAssetPageList(GenericPageParam genericPageParam,
                                                @RequestParam(value = "videoContentCatId", required = false) String videoContentCatId,
                                                @RequestParam(value = "videoContentCopyrightOwnerId", required = false) String videoContentCopyrightOwnerId) {
        LambdaQueryWrapper<Asset> ew = new LambdaQueryWrapper<>();
        // 对名称或编码模糊查询
        if (StringUtils.isNotBlank(genericPageParam.getNameOrCode())) {
            ew.like(Asset::getFileName, genericPageParam.getNameOrCode());
        }
        // 设置排序规则
        ew.orderByDesc(Asset::getCreateTime);
        Page<Asset> page = this.assetService.pageByConditions(getPage(), videoContentCatId, videoContentCopyrightOwnerId);
        return getResult(page);
    }

    @ApiOperation(value = "获取视频文件详情", notes = "获取视频文件详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "标识ID", dataType = "String", paramType = "path")
    })
    @GetMapping("/get/{id}")
    @RequiresAuthentication  //@RequiresPermissions("asset:get:id")
    public Map<String, Object> getById(@PathVariable("id") String id) {
        Asset asset = assetService.getById(id);
        return getResult(asset);
    }

    @ApiOperation(value = "视频文件汇出", notes = "视频文件汇出")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "idList", value = "视频文件标识ID集合", dataType = "String", paramType = "path")
    })
    @PostMapping("/download")
    //@RequiresAuthentication
    public void download(@RequestParam("idList") List<String> idList, HttpServletResponse response) throws IOException {
        response.setContentType("application/octet-stream");
        //保证下载时文件名为指定文件名，避免中文文件名乱码
        response.addHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + URLEncoder.encode("1.zip", "UTF-8") + "\"");

        final List<Asset> assetList = assetService.listByIds(idList);
        final InputStream[] inputStreams = new InputStream[idList.size()];
        final String[] paths = new String[idList.size()];
        for (int i = 0; i < assetList.size(); i++) {
            final Asset asset = assetList.get(i);
            final String fileUrl = asset.getFileUrl();
            ByteOutputStream byteOutputStream = new ByteOutputStream();
            FastDFSUtils.downloadFile(fileUrl, byteOutputStream);
            inputStreams[i] = VideoEncryptUtil.encrypt(byteOutputStream.newInputStream(), VideoEncryptUtil.cipher);
            final Set<MetaData> fileMetaData = FastDFSUtils.getFileMetaData(fileUrl);
            String md5 = fileMetaData.stream().filter(x -> "MD5".equals(x.getName())).map(MetaData::getValue).findFirst().get();
            paths[i] = md5 + ".chnmuseum";
        }
        ServletOutputStream outputStream = response.getOutputStream();
        ZipUtil.zip(outputStream, paths, inputStreams);
    }

    @ApiOperation(value = "视频文件汇出", notes = "视频文件汇出")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "idList", value = "视频文件标识ID集合", dataType = "String", paramType = "path")
    })
    @GetMapping("/download1")
    //@RequiresAuthentication
    public void download1(@RequestParam("idList") List<String> idList, HttpServletResponse response) throws IOException {
        response.setContentType("application/octet-stream");
        //保证下载时文件名为指定文件名，避免中文文件名乱码
        response.addHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + URLEncoder.encode("1.zip", "UTF-8") + "\"");

        final List<Asset> assetList = assetService.listByIds(idList);
        final InputStream[] inputStreams = new InputStream[idList.size()];
        final String[] paths = new String[idList.size()];
        for (int i = 0; i < assetList.size(); i++) {
            final Asset asset = assetList.get(i);
            final String fileUrl = asset.getFileUrl();
            ByteOutputStream byteOutputStream = new ByteOutputStream();
            FastDFSUtils.downloadFile(fileUrl, byteOutputStream);
            inputStreams[i] = VideoEncryptUtil.encrypt(byteOutputStream.newInputStream(), VideoEncryptUtil.cipher);
            final Set<MetaData> fileMetaData = FastDFSUtils.getFileMetaData(fileUrl);
            String md5 = fileMetaData.stream().filter(x -> "MD5".equals(x.getName())).map(MetaData::getValue).findFirst().get();
            paths[i] = md5 + ".chnmuseum";
        }
        ServletOutputStream outputStream = response.getOutputStream();
        ZipUtil.zip(outputStream, paths, inputStreams);
    }

    @ApiOperation(value = "视频文件汇出", notes = "视频文件汇出")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "idList", value = "视频文件标识ID集合", dataType = "String", paramType = "path")
    })
    @GetMapping("/download2")
    //@RequiresAuthentication
    public void download2(@RequestParam("idList") List<String> idList, HttpServletResponse response) throws IOException {
        response.setContentType("application/octet-stream");
        //保证下载时文件名为指定文件名，避免中文文件名乱码
        response.addHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + URLEncoder.encode("1.zip", "UTF-8") + "\"");

        final Map<String, InputStream> map = new LinkedHashMap<>(idList.size() + 1);
        final List<Asset> assetList = assetService.listByIds(idList);
        final String cipher = RandomUtil.randomString(BASE_STRING, 16);
        for (final Asset asset : assetList) {
            final String fileUrl = asset.getFileUrl();
            ByteArrayOutputStream byteOutputStream = new ByteArrayOutputStream();
            FastDFSUtils.downloadFile(fileUrl, byteOutputStream);
            final Set<MetaData> fileMetaData = FastDFSUtils.getFileMetaData(fileUrl);
            String fileName = fileMetaData.stream().filter(x -> "MD5".equals(x.getName())).map(MetaData::getValue).findFirst().get() + ".chnmuseum";
            map.put(fileName, VideoEncryptUtil.encrypt(new ByteArrayInputStream(byteOutputStream.toByteArray()), cipher));
        }
        map.put("23432423432.key", null);



        ServletOutputStream outputStream = response.getOutputStream();
        String[] paths = map.keySet().toArray(new String[0]);
        InputStream[] ins = map.values().toArray(new InputStream[0]);
        ZipUtil.zip(outputStream, paths, ins);
    }

}

